/**************************************************************************
 * HeXHub Messenger Plugin Copyright (C) 2007-2008 Mikhail Polenin        *
 * HeXHub is Copyright(C) Albu Cristian Laurentiu                         *
 * This plugin is licensed under the Open Software License 3.0            *
 * Please read the license before making a modification to this plugin.   *
 * Failure to follow the terms of the license will have severe            *
 * consequences                                                           *
 **************************************************************************/

#include "listclass.h"

float pluginversion = 0.3;
extern char dir[MAX_PATH];
extern HANDLE hQueue;
extern VOID CALLBACK OnRemind(PVOID thisbox, BOOL TimerWaitOrFired);
extern const char *botnick;

Mailbox::Mailbox()
{
	this->first = 0;
	this->last = 0;
	this->total = 0;
	InitializeCriticalSection(&mbcs);
}

Mailbox::~Mailbox(void)
{
	EnterCriticalSection(&mbcs);
	box *rem = this->first;
	box *forwardpos;
	while(rem != 0)
	{
		forwardpos = rem->next;
		DeleteTimerQueueTimer(NULL, rem->Tmr, NULL);
		delete rem;
		rem = forwardpos;
	}
	LeaveCriticalSection(&mbcs);
	DeleteCriticalSection(&mbcs);
}

std::string Mailbox::GetFileSafeString(char *str)
{
	std::string fsstr(str);
	std::string::size_type st;
	st = fsstr.find('\\', 0);
	if(st != std::string::npos)
		fsstr.replace(st, 1, "$bsl;");
	st = fsstr.find('/', 0);
	if(st!= std::string::npos)
		fsstr.replace(st, 1, "$fsl;");
	st = fsstr.find(':', 0);
	if(st != std::string::npos)
		fsstr.replace(st, 1, "$cln;");
	st = fsstr.find('*', 0);
	if(st != std::string::npos)
		fsstr.replace(st, 1, "$atk;");
	st = fsstr.find('?', 0);
	if(st != std::string::npos)
		fsstr.replace(st, 1, "$qtm;");
	st = fsstr.find('\"', 0);
	if(st != std::string::npos)
		fsstr.replace(st, 1, "$qot;");
	st = fsstr.find('<', 0);
	if(st != std::string::npos)
		fsstr.replace(st, 1, "$lt;");
	st = fsstr.find('>', 0);
	if(st != std::string::npos)
		fsstr.replace(st, 1, "$gt;");
	return fsstr;
}

std::string Mailbox::GetNickFromFileSafeString(char *str)
{
	std::string fsstr(str);
	std::string::size_type st;
	st = fsstr.find("$bsl;", 0);
	if(st != std::string::npos)
		fsstr.replace(st, 1, "\\");
	st = fsstr.find("$fsl;", 0);
	if(st!= std::string::npos)
		fsstr.replace(st, 1, "/");
	st = fsstr.find("$cln;", 0);
	if(st != std::string::npos)
		fsstr.replace(st, 1, ":");
	st = fsstr.find("$atk;", 0);
	if(st != std::string::npos)
		fsstr.replace(st, 1, "*");
	st = fsstr.find("$qtm;", 0);
	if(st != std::string::npos)
		fsstr.replace(st, 1, "?");
	st = fsstr.find("$qot;", 0);
	if(st != std::string::npos)
		fsstr.replace(st, 1, "\"");
	st = fsstr.find("$lt;", 0);
	if(st != std::string::npos)
		fsstr.replace(st, 1, "<");
	st = fsstr.find("$gt;", 0);
	if(st != std::string::npos)
		fsstr.replace(st, 1, ">");
	return fsstr;
}

BOOL Mailbox::Add(char *nick)
{
	EnterCriticalSection(&mbcs);
	std::stringstream folder;

	folder << dir << "\\Plugins\\Messages\\" << GetFileSafeString(nick);

	box *newnode = new box;
	strcpy(newnode->nick, nick);

	if((newnode->Remind == TRUE) && (newnode->ReminderTime > 0))
	{
		CreateTimerQueueTimer(&newnode->Tmr, NULL, (WAITORTIMERCALLBACK)OnRemind, newnode,
			newnode->ReminderTime*60*1000, newnode->ReminderTime*60*1000, WT_EXECUTEINTIMERTHREAD);
	}

	if (this->first == 0) {
		this->first = newnode;
		this->last = newnode;
	} else {
		this->last->next = newnode;
		this->last = newnode;
	}
	CreateDirectory(folder.str().c_str(), NULL);
	
	this->total++;
	LeaveCriticalSection(&mbcs);
	return true;

}

BOOL Mailbox::Add(secstruct temp)
{
	EnterCriticalSection(&mbcs);

	box *newnode = new box;
	memcpywithnull(newnode->nick, temp.nick, 40);
	newnode->amtofmessages = temp.AmtofMessages;
	newnode->PMNotify = temp.PMNotify;
	newnode->ReminderTime = temp.ReminderTime;
	newnode->Remind = temp.Remind;

	if((newnode->Remind == TRUE) && (newnode->ReminderTime > 0))
	{
		CreateTimerQueueTimer(&newnode->Tmr, NULL, (WAITORTIMERCALLBACK)OnRemind, newnode,
			newnode->ReminderTime*60*1000, newnode->ReminderTime*60*1000, WT_EXECUTEINTIMERTHREAD);
	}

	if (this->first == 0) {
		this->first = newnode;
		this->last = newnode;
	} else {
		this->last->next = newnode;
		this->last = newnode;
	}
	
	this->total++;
	LeaveCriticalSection(&mbcs);
	return true;

}

BOOL Mailbox::Find(char *nick)
{
	EnterCriticalSection(&mbcs);

	std::stringstream folder;

	folder << dir << "\\Plugins\\Messages\\" << GetFileSafeString(nick);

	if(!CreateDirectory(folder.str().c_str(), NULL))
	{
		LeaveCriticalSection(&mbcs);
		return true;
	}

	LeaveCriticalSection(&mbcs);
	RemoveDirectory(dir);
	return false;
}

box *Mailbox::FindBox(char *nick)
{
	EnterCriticalSection(&mbcs);

	box *pos = this->first;
	box *forward;
	while(pos != 0)
	{
		forward = pos->next;
		if(strcmp(pos->nick, nick) == 0)
		{
			LeaveCriticalSection(&mbcs);
			return pos;
		}
		pos = forward;
	}

	LeaveCriticalSection(&mbcs);
	return 0;
}

BOOL Mailbox::AppendMsg(std::string msg, char *nick, char *fromnick)
{
	EnterCriticalSection(&mbcs);
	box *pos = this->first;
	box *forward;
	while(pos != 0)
	{
		forward = pos->next;
		if(strcmp(pos->nick, nick) == 0)
		{
			pos->amtofmessages++;
			std::stringstream filename;
			filename << dir << "\\Plugins\\Messages\\" << GetFileSafeString(nick) << '\\' << pos->amtofmessages << ".txt";
			HANDLE hFile = CreateFile(filename.str().c_str(), GENERIC_WRITE, FILE_SHARE_READ, NULL,
				CREATE_ALWAYS, NULL, NULL);

			if(hFile != INVALID_HANDLE_VALUE)
			{
				std::stringstream buffer;
				buffer << "Message Number " << pos->amtofmessages << " From: " << fromnick << ": " << msg << "\r\n";

				DWORD numwritten = 0;
				WriteFile(hFile, buffer.str().c_str(), buffer.str().length(), &numwritten, NULL);
				CloseHandle(hFile);
				LeaveCriticalSection(&mbcs);
				return true;
			}
			else
			{
				CloseHandle(hFile);
				LeaveCriticalSection(&mbcs);
				return false;
			}
		}
		pos = forward;
	}
	LeaveCriticalSection(&mbcs);
	return false;
}

std::string Mailbox::GetMessages(char *nick)
{
	EnterCriticalSection(&mbcs);
	box *pos = this->first;
	box *forward;

	while(pos != 0)
	{
		forward = pos->next;
		if(strcmp(pos->nick, nick) == 0)
		{
			std::stringstream buff;
			buff << "\r\n Your Messages: \r\n";

			int i = 0;
			std::string nck = GetFileSafeString(nick);

			for(i = 0; i <= pos->amtofmessages; i++)
			{
				std::stringstream filename;
				filename << dir << "\\Plugins\\Messages\\" << nck << '\\' << i << ".txt";
				HANDLE hFile = CreateFile(filename.str().c_str(), GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING,
					NULL, NULL);
				if(hFile != INVALID_HANDLE_VALUE)
				{
					DWORD NumRead = 0;
					char buffer[32768];
					memset(buffer, 0, 32768);
					ReadFile(hFile, buffer, 32768, &NumRead, NULL);
					buff << buffer;
					CloseHandle(hFile);
				}
				if(i == pos->amtofmessages) break;
			}
			LeaveCriticalSection(&mbcs);
			buff << "\r\n\r\n\t You may clear your mailbox at any time by using !delmsgs.";
			return buff.str().c_str();
		}
		pos = forward;
	}
	LeaveCriticalSection(&mbcs);
	return "Error: Could not retrieve your mailbox.";
}

BOOL Mailbox::ClearMailbox(char *nick)
{
	EnterCriticalSection(&mbcs);
	box *pos = this->first;
	box *forward;
	while(pos != 0)
	{
		forward = pos->next;
		if(strcmp(pos->nick, nick) == 0)
		{

			int amt = pos->amtofmessages;
			int i = 0;
			std::string nck = GetFileSafeString(nick);

			for(i = 0; i <= amt; i++)
			{
				std::stringstream filename;
				filename << dir << "\\Plugins\\Messages\\" << nck << '\\' << i << ".txt";
				if(DeleteFile(filename.str().c_str())) pos->amtofmessages--;
				if(i == amt) break;
			}
			LeaveCriticalSection(&mbcs);
			return true;
		}
		pos = forward;
	}
	LeaveCriticalSection(&mbcs);
	return false;
}

BOOL Mailbox::SaveConfig()
{
	EnterCriticalSection(&mbcs);

	DWORD NumWritten = 0;

	std::stringstream filename;
	filename << dir << "\\Plugins\\" << "messenger.dat";


	HANDLE hFile;

	if((hFile = CreateFile(filename.str().c_str(), GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, 
		FILE_ATTRIBUTE_NORMAL, NULL)) == INVALID_HANDLE_VALUE){ LeaveCriticalSection(&mbcs); return false; }

	WriteFile(hFile, &pluginversion, sizeof(float), &NumWritten, NULL);


	box *pos = this->first;
	box *forward;
	while(pos != 0)
	{
		secstruct temp;
		forward = pos->next;
		strcpy_s(temp.nick, 40, pos->nick);
		temp.AmtofMessages = pos->amtofmessages;
		temp.PMNotify = pos->PMNotify;
		temp.Remind = pos->Remind;
		temp.ReminderTime = pos->ReminderTime;
		WriteFile(hFile, &temp, sizeof(secstruct), &NumWritten, NULL);
		pos = forward;
	}
	LeaveCriticalSection(&mbcs);
	CloseHandle(hFile);
	return true;
}

BOOL Mailbox::LoadConfig()
{
	EnterCriticalSection(&mbcs);
	float ver = 0.0;
	std::stringstream folder;

	folder << dir << "\\Plugins\\Messages\\";

	CreateDirectory(folder.str().c_str(), NULL);

	std::stringstream filename;
	filename << dir << "\\Plugins\\" << "messenger.dat";

	LONG offSet = 0;
	DWORD bytesRead = 1;
	DWORD newpos = 0;
	DWORD getl = 0;
	BOOL chk = true;

	HANDLE hFile;

	if((hFile = CreateFile(filename.str().c_str(), GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 
		FILE_ATTRIBUTE_NORMAL, NULL)) == INVALID_HANDLE_VALUE) { LeaveCriticalSection(&mbcs); return false; }

	ReadFile(hFile, &ver, sizeof(float), &bytesRead, NULL);

	secstruct temp;
	while(chk == true && bytesRead != 0)
	{
		chk = ReadFile(hFile, &temp, MAX_READ, &bytesRead, NULL);
		getl = GetLastError();
		offSet += MAX_READ;
		this->Add(temp);
	}
	CloseHandle(hFile);

	LeaveCriticalSection(&mbcs);
	return true;
}