/**************************************************************************
 * HeXHub Messenger Plugin Copyright (C) 2007-2008 Mikhail Polenin        *
 * HeXHub is Copyright(C) Albu Cristian Laurentiu                         *
 * This plugin is licensed under the Open Software License 3.0            *
 * Please read the license before making a modification to this plugin.   *
 * Failure to follow the terms of the license will have severe            *
 * consequences                                                           *
 **************************************************************************/
#define  _WIN32_WINNT 0x0500
#include <string>
#include <sstream>
#include "HeXHubSDK.h"
#include "listclass.h"

const char *botnick = "Hermes";

long trigId, hlpId, msgchktrigId, msgchkhlpId, msgdltrigId, msgdlhlpId;
long opttrigId, opthlpId;
long botId;
Mailbox *Msgbox = new Mailbox;
CRITICAL_SECTION cs;
char dir[MAX_PATH];

void HEXHUBAPI onMessageCommand(SDWORD dwCommandId, SDWORD userId, LPVOID rights, const char *commandParameters, BOOL isPM);
void HEXHUBAPI onChkCommand(SDWORD dwCommandId, SDWORD userId, LPVOID rights, const char *commandParameters, BOOL isPM);
void HEXHUBAPI onDelCommand(SDWORD dwCommandId, SDWORD userId, LPVOID rights, const char *commandParameters, BOOL isPM);
void HEXHUBAPI onMsgrCommand(SDWORD dwCommandId, SDWORD userId, LPVOID rights, const char *commandParameters, BOOL isPM);
VOID CALLBACK OnRemind(PVOID thisbox, BOOL TimerWaitOrFired);
void HEXHUBAPI onBotPM(SDWORD dwBotId,SDWORD userId,LPVOID lpMessage,int bufferSize);

const char * HEXHUBAPI InitPlugin(lpfnGetFunction HexGetFunction) {
	HexHub Hex;
	
	if (Hex.Initialize(HexGetFunction) == false) return 0;

	if ((msgchktrigId = HexHub::Core::RegisterCommand("checkmsgs", onChkCommand)) == -1) return 0;
	if ((msgdltrigId = HexHub::Core::RegisterCommand("delmsgs", onDelCommand)) == -1) return 0;
	if ((trigId = HexHub::Core::RegisterCommand("message", onMessageCommand)) == -1) return 0;
	if ((opttrigId = HexHub::Core::RegisterCommand("messenger", onMsgrCommand)) == -1) return 0;
	if ((botId = HexHub::Core::RegisterBot(botnick, (FARPROC)onBotPM)) == -1) return 0;
	
	
	if ((hlpId = HexHub::Core::RegisterHelpString("EN", "!message <username> <body>\t\t\t-\tSend A Message to a user via Messenger Plugin", 1)) == -1) {
		HexHub::Core::UnregisterCommand(trigId);
		return 0;
	}

	if ((msgchkhlpId = HexHub::Core::RegisterHelpString("EN", "!checkmsgs\t\t\t-\tCheck your messages", 1)) == -1) {
		HexHub::Core::UnregisterCommand(msgchktrigId);
		return 0;
	}

	if ((msgdlhlpId = HexHub::Core::RegisterHelpString("EN", "!delmsgs\t\t\t-\tDelete your messages", 1)) == -1) {
		HexHub::Core::UnregisterCommand(msgchktrigId);
		return 0;
	}

	if ((opthlpId = HexHub::Core::RegisterHelpString("EN", "!messenger\t\t\t-\tSet Messenger Plugin-Related options", 1)) == -1) {
		HexHub::Core::UnregisterCommand(trigId);
		return 0;
	}

	GetModuleFileName(NULL, dir, sizeof(dir));

	int i = strlen(dir);

	for(i = strlen(dir); i >= 0; i--)
	{
		if(dir[i] == '\\')
		{
			dir[i+1] = 0;
			break;
		}
	}

	InitializeCriticalSection(&cs);

	Msgbox->LoadConfig();
	return "Messenger Plugin 0.3 by The Architect";
}

void HEXHUBAPI onNicklistChange(SDWORD dwUserId,BOOL addedToList)
{
	int AccId = 0;
	char btnck[40];
	if(!HexHub::Users::NickFromId(botId, btnck)) return;
	if((AccId = HexHub::Users::Registration::GetAccountId(dwUserId)) > 0 && addedToList == true)
	{
		char Nck[40];
		if(HexHub::Users::ClientNickFromId(dwUserId, Nck))
		{
			EnterCriticalSection(&cs);
			if(!Msgbox->Find(Nck))
			{
				Msgbox->Add(Nck);
				std::stringstream Sendstr;
				Sendstr << "<" << btnck << "> \r\n\r\n\tA mailbox has been added for you. You can check your messages with !checkmsgs :)"
					<< "\r\n\tTo message someone, type !message <nickname> <body>\r\n\tTo clear your mailbox, type !delmsgs|";
				HexHub::Data::SendTo(dwUserId, Sendstr.str().c_str(), Sendstr.str().length());
			}
			else
			{
				box *val = 0;
				std::stringstream Sendstr;
				if((val = Msgbox->FindBox(Nck)) != 0)
				{
					Sendstr << "<" << btnck << "> You have " << val->amtofmessages << " messages in your mailbox. Please type !checkmsgs to check your messages.|";
					HexHub::Data::SendTo(dwUserId, Sendstr.str().c_str(), Sendstr.str().length());
				}
			}
			LeaveCriticalSection(&cs);
		}
	}
}

void HEXHUBAPI onMessageCommand(SDWORD dwCommandId, SDWORD userId, LPVOID rights, const char *commandParameters, BOOL isPM)
{
	char SenderNick[41];
	char *body = 0;
	char btnck[40];
	if(!HexHub::Users::NickFromId(botId, btnck)) return;
	if(HexHub::Users::ClientNickFromId(userId, SenderNick))
	{
		char Parameters[8192];
		int i = 0;
		memset(Parameters, 0, 8192);
		for (i=0; (i < 8191) && (commandParameters[i] != '|') && (commandParameters[i] != 0); i++) Parameters[i] = commandParameters[i];
		for (i=0; Parameters[i] != 0; i++) {
			if (Parameters[i] == ' ') {
				Parameters[i] = 0;
				body = (&Parameters[i]) + 1;
				break;
			}
		}
		std::stringstream SendValid;
		if(Parameters[0] != 0)
		{
			if(body != 0)
			{
				SendValid << "<" << btnck << "> You have chosen to message " << Parameters << " with the body: " << body << "|";
				HexHub::Data::SendTo(userId, SendValid.str().c_str(), SendValid.str().length());
				SendValid.str("");
				EnterCriticalSection(&cs);
				if(Msgbox->Find(Parameters))
				{
					if(Msgbox->AppendMsg(body, Parameters, SenderNick))
					{
						SendValid << "<" << btnck << "> Send to " << Parameters << " was successful!|";

						int id = 0;
						if((id = HexHub::Users::IdFromNick(Parameters)) > -1)
						{
							box *var = Msgbox->FindBox(Parameters);
							if(var != 0)
							{
								if(var->PMNotify)
								{
									std::stringstream snd;
									snd << "$To: " << Parameters << " From: " << btnck << " $<" << btnck << "> You have a new message from " << SenderNick << " . Please type !checkmsgs to check your messages!|";
									HexHub::Data::SendTo(id, snd.str().c_str(), snd.str().length());
								}
								else
								{
									std::stringstream snd;
									snd << "<" << btnck << "> You have a new message from " << SenderNick << " . Please type !checkmsgs to check your messages!|";
									HexHub::Data::SendTo(id, snd.str().c_str(), snd.str().length());
								}
							}
						}
					}
					else
					{
						SendValid << "<" << btnck << "> Send to " << Parameters << " failed, for some odd reason.|";
					}
					HexHub::Data::SendTo(userId, SendValid.str().c_str(), SendValid.str().length());
				}
				else
				{
					SendValid << "<" << btnck << "> Mailbox for " << Parameters << " does not exist. Sorry.|";
					HexHub::Data::SendTo(userId, SendValid.str().c_str(), SendValid.str().length());
				}
				LeaveCriticalSection(&cs);
			}
			else
			{
				SendValid << "<" << btnck << "> You must specify a message.|";
				HexHub::Data::SendTo(userId, SendValid.str().c_str(), SendValid.str().length());
			}
		}
		else
		{
			SendValid << "<" << btnck << "> You must specify a Nick to send to!|";
			HexHub::Data::SendTo(userId, SendValid.str().c_str(), SendValid.str().length());
		}
	}
}

const char * HEXHUBAPI GetAboutText(void) {
	return "Mikhail Polenin, 2007 - 2008, Release date: " __DATE__ " " __TIME__ "";
}

void HEXHUBAPI SaveSettings()
{
	Msgbox->SaveConfig();
}

void HEXHUBAPI UnloadPlugin()
{
	delete Msgbox;
	if(botId > -1)
	{ HexHub::Core::UnregisterBot(botId); }
	if(msgchktrigId > -1) HexHub::Core::UnregisterCommand(msgchktrigId);
	if(msgdltrigId > -1) HexHub::Core::UnregisterCommand(msgdltrigId);
	if(trigId > -1) HexHub::Core::UnregisterCommand(trigId);
	if(opttrigId > -1) HexHub::Core::UnregisterCommand(opttrigId);
	if(hlpId > -1) HexHub::Core::UnregisterHelpString(hlpId);
	if(msgchkhlpId > -1) HexHub::Core::UnregisterHelpString(msgchkhlpId);
	if(msgdlhlpId > -1) HexHub::Core::UnregisterHelpString(msgdlhlpId);
	if(opthlpId > -1) HexHub::Core::UnregisterHelpString(opthlpId);
}

void HEXHUBAPI onChkCommand(SDWORD dwCommandId, SDWORD userId, LPVOID rights, const char *commandParameters, BOOL isPM)
{
	char btnck[40];
	if(!HexHub::Users::NickFromId(botId, btnck)) return;
	char usernick[40];
	HexHub::Users::ClientNickFromId(userId, usernick);
	EnterCriticalSection(&cs);
	if(Msgbox->Find(usernick))
	{
		std::stringstream Sendstr;
		Sendstr << "<" << btnck << "> " << Msgbox->GetMessages(usernick) << "|";
		HexHub::Data::SendTo(userId, Sendstr.str().c_str(), Sendstr.str().length());
	}
	else
	{
		std::stringstream str;
		str << "<" << btnck << "> You do not have a mailbox.|";
		HexHub::Data::SendTo(userId, str.str().c_str(), str.str().length());
	}
	LeaveCriticalSection(&cs);
}

void HEXHUBAPI onDelCommand(SDWORD dwCommandId, SDWORD userId, LPVOID rights, const char *commandParameters, BOOL isPM)
{
	char usernick[40];
	char btnck[40];
	if(!HexHub::Users::NickFromId(botId, btnck)) return;
	HexHub::Users::ClientNickFromId(userId, usernick);
	EnterCriticalSection(&cs);
	if(Msgbox->Find(usernick))
	{
		std::stringstream Sendstr;
		if(Msgbox->ClearMailbox(usernick))
		{
			Sendstr << "<" << btnck << "> Successfully Cleared Your Mailbox!|";
			HexHub::Data::SendTo(userId, Sendstr.str().c_str(), Sendstr.str().length());
		}
		else
		{
			Sendstr << "<" << btnck << "> Error Clearing Mailbox!|";
			HexHub::Data::SendTo(userId, Sendstr.str().c_str(), Sendstr.str().length());
		}
	}
	else
	{
		std::stringstream str;
		str << "<" << btnck << "> You do not have a mailbox.|";
		HexHub::Data::SendTo(userId, str.str().c_str(), str.str().length());
	}
	LeaveCriticalSection(&cs);
}

void HEXHUBAPI onMsgrCommand(SDWORD dwCommandId, SDWORD userId, LPVOID rights, const char *commandParameters, BOOL isPM)
{
	char btnck[40];
	if(!HexHub::Users::NickFromId(botId, btnck)) return;
	char usernick[40];
	HexHub::Users::ClientNickFromId(userId, usernick);
	box *var = 0;
	std::string context;

	char Parameter[8192];
	memset(Parameter, 0, 8192);
	int i = 0;
	for (i=0; (i < 8191) && (commandParameters[i] != '|') && (commandParameters[i] != 0); i++) Parameter[i] = commandParameters[i];
	for (i=0;Parameter[i] != 0; i++) {
		if (Parameter[i] == ' ') {
			Parameter[i] = 0;
			context = &Parameter[i+1];
			break;
		}
	}

	std::stringstream buff;
	std::string ParamVal;

	EnterCriticalSection(&cs);
	if((var = Msgbox->FindBox(usernick)) != 0)
	{
		if(strcmp("pm", Parameter) == 0)
		{
			if(strcmp("on", context.c_str()) == 0)
			{
				var->PMNotify = 1;
				buff << "*** Configuration Changed.|";
			}
			else if(strcmp("off", context.c_str()) == 0)
			{
				var->PMNotify = 0;
				buff << "*** Configuration Changed.|";
			}
			else
			{
				if(var->PMNotify) ParamVal = "on";
				else ParamVal = "off";
				buff << "*** Current values for pm are\r\n\t"
					<< "\r\n\r\n\t\tBoolean = \"" << ParamVal << "\"|";
			}
		}
		else if(strcmp("remind", Parameter) == 0)
		{
			int iz0r = 0;
			std::stringstream converter(context);
			converter >> iz0r;
			if(strcmp("on", context.c_str()) == 0)
			{
				var->Remind = 1;
				buff << "*** Configuration Changed.|";
			}
			else if(strcmp("off", context.c_str()) == 0)
			{
				var->Remind = 0;
				buff << "*** Configuration Changed.|";
			}
			else
			{
				if(iz0r > 0)
				{
					DeleteTimerQueueTimer(NULL, var->Tmr, NULL);
					var->ReminderTime = iz0r;
					CreateTimerQueueTimer(&var->Tmr, NULL, (WAITORTIMERCALLBACK)OnRemind, var,
						var->ReminderTime*60*1000, var->ReminderTime*60*1000, WT_EXECUTEINTIMERTHREAD);
					buff << "*** Configuration Changed.|";
				}
				else
				{
					if(var->Remind) ParamVal = "on";
					else ParamVal = "off";
					buff << "*** Current values for remind are\r\n\t"
						<< "\r\n\r\n\t\tBoolean = \"" << ParamVal << "\"\r\n\t\tInt32 = \"" << var->ReminderTime << "\"|";
				}
			}
		}
		else
		{
			buff << "*** You have written an invalid parameter for !messenger [param]\r\n\t"
				<< "Valid values for !messenger [param] are: \r\n\r\n\t\tpm (Notification of new messages in your PM window on connect)"
				<< "\r\n\t\tremind (Reminder settings)|";
		}
	}
	else
	{
		buff << "*** You must have a mailbox to use this command. Only registered users may have mailboxes.|";
	}
	LeaveCriticalSection(&cs);
	HexHub::Data::SendTo(userId, buff.str().c_str(), buff.str().length());
}

void HEXHUBAPI onBotPM(SDWORD dwBotId,SDWORD userId,LPVOID lpMessage,int bufferSize)
{
	char btnck[40];
	if(!HexHub::Users::NickFromId(botId, btnck)) return;
	char nick[40];
	HexHub::Users::NickFromId(userId, nick);
	std::stringstream snd;
	snd << "Conversation with " << btnck << ": " << (char *)lpMessage;
	HexHub::Data::SendNotificationMessage(snd.str().c_str());
}

VOID CALLBACK OnRemind(PVOID thisbox, BOOL TimerWaitOrFired)
{
	char btnck[40];
	if(!HexHub::Users::NickFromId(botId, btnck)) return;
	box *user = (box *)thisbox;
	SDWORD userId = 0;
	if((user->Remind) && (user->amtofmessages > 0) && ((userId = HexHub::Users::IdFromNick(user->nick)) > -1))
	{
		std::stringstream snd;
		if(user->PMNotify == true)
		{
			snd << "$To: " << user->nick << " From: " << btnck << " $<" << btnck << "> You have " 
				<< user->amtofmessages << " in your mailbox. Please write !checkmsgs in mainchat to check these messages."
				<< "\r\n\tWrite !messenger remind off in mainchat to turn off reminders. Write !messenger pm off in mainchat to turn off PM Notifications.|";
		}
		else
		{
			snd << "<" << btnck << "> You have " << user->amtofmessages << " in your mailbox. Please write !checkmsgs in mainchat to check these messages."
				<< "\r\n\tWrite !messenger remind off to turn off reminders.|";
		}
		HexHub::Data::SendTo(userId, snd.str().c_str(), snd.str().length());
	}
}